/* 
LFO generates a continuous ramp of numbers for a specified range, offset, duration and mode.
arguments:
<min>, <max>, <duration in ms>,<mode 0=saw, 1=sine, 2=triangle, 3=squarewave>, <0=float output, 1=integer output>, <invert>
if only one is specified, that is the max value, and lfo will sweep from 0 to max
*/
autowatch = 1;

inlets = 5;
setinletassist(0,"on/off,messages");
setinletassist(1,"min");
setinletassist(2,"max");
setinletassist(3,"duration");
setinletassist(4,"mode");
outlets = 2;
setoutletassist(0,"lfo output");
setoutletassist(1,"lfo end bang");

var tog=0;
var taskinterval = 20;
var taskintervalhold = 20;
var lfo = new Task(calclfo,this);
lfo.interval = taskinterval;
var clock = 0;
var dur = 1000;
var rmin = 0.;
var rmax = 1.;
var intmode = 0;
var sinemode = 0.;
var cosmode = 0.;
var mode = 0;
var invertit = 0;
var nrt = 0;
var fps = 30.;
var eventcount = 0;
const PI = 3.14159;
  
// process arguments
if (jsarguments.length==2) //if only 1 arg is given, it is max value
	rmax = jsarguments[1];
if (jsarguments.length>2)
	rmin = jsarguments[1];
	rmax = jsarguments[2];
if (jsarguments.length>3)
	dur = jsarguments[3];
if (jsarguments.length>4)
	mode = jsarguments[4];
if (jsarguments.length>5)
	intmode = jsarguments[5];
if (jsarguments.length>6)
	invertit = jsarguments[6];

function reset(){
	clock=0;
}

function msg_float(v){
	if (inlet == 1) rmin = v;
	if (inlet == 2) rmax = v;	
	if (inlet == 3) dur = v;	
	if (inlet == 4) mode = Math.floor(v);	
}
function msg_int(v){
	if (inlet == 0) onoff(v);
	if (inlet == 1) rmin = v;
	if (inlet == 2) rmax = v;	
	if (inlet == 3) dur = v;	
	if (inlet == 4) mode = v;	
}
function rate(v){
	dur = v;
}

var rndB=Math.random();
var rndA = rndB;
var rndrange = 1.;
var flip = 0;
function calclfo(){
	var lfoout;
	var time = ((taskinterval*clock)%dur);
	//post("\ntime",time);
	var src = time/dur;
	var range = rmax-rmin;

	lfoout = (src*range)+rmin;
	switch(mode){
		case 0: //sawtooth
			lfoout = (src*range)+rmin;
		break;
		case 1: //sine
			lfoout = (range*Math.sin(src*PI))+rmin;
		break;
		case 2:  // triangle
			var halfway = dur/2;			
			if (time<halfway) {
				lfoout = (2*src*range)+rmin;
			}else{
				var srctmp = (halfway-(time-halfway))/dur;
				lfoout = (2*srctmp*range)+rmin;
			}
		break;
		case 3: //square waver
			var low = time<dur/2;
			var hi = time>=dur/2;
			lfoout = low*rmin+hi*rmax;
		break;
		case 4: //random
		if (time >= dur-taskinterval) rndA = Math.random();		
			lfoout = (rndA*range)+rmin;
		break;
		case 5:  // random triangle aka continuous random aka randangle
		if (time < taskinterval) {
			rndA = rndB;
			rndB = range*Math.random()+rmin;
			rndrange = rndB-rndA;
		}
		lfoout = src*rndrange+rndA;
		break;
	}

		if (intmode) lfoout = Math.floor(lfoout);
	
	if(!invertit)
		outlet (0,lfoout);
	else
		outlet (0,rmax-lfoout+rmin);
	if (time >= dur-taskinterval) outlet(1,"bang");
	clock++;
}
//calclfo.immediate = 1;

function onoff(v){
	//lfo.interval = taskinterval;
	if (v){ 
		if(!nrt) lfo.repeat();
	}else{
		lfo.cancel();
	}
	tog = v;
}

function bang(){
	if(tog&&nrt) calclfo();
}
 
function range(a,b){
	if(a<b){
		rmin = a;
		rmax = b;
	}else{
		rmin = b;
		rmax = a;
	}
}

function toint(v){
	intmode = v;
}

function outmode(v){ //saw ,sine, tri
	mode = v;
}

function outmodemenu(v){ //saw ,sine, tri
	mode = v-1;
	onoff(v>0);
}

function invert(v){
	invertit = v;
}
function grain(v){	
	taskinterval = v;
	lfo.interval = taskinterval;
	/*
	if(lfo.running) {
		onoff(1)
	}
	*/
}

function eventmode(v){
	nrt = v;
	if(v) {
		taskintervalhold = taskinterval;
		if(lfo.running) lfo.cancel();
		taskinterval = 1000/fps; 
	}else{
		if(tog) lfo.repeat();
		taskinterval = taskintervalhold;
	}
}
 
function framerate(v){
	fps = v;
	if(nrt) taskinterval = 1000/fps; 
}